 precision highp float;
 varying highp vec2 textureCoordinate;
 varying highp vec2 textureCoordinate2;
 uniform highp float gradientPosStart;
 uniform highp float gradientPosEnd;

 uniform highp sampler2D inputImageTexture;
 uniform highp sampler2D inputImageTexture2;
 uniform highp vec3 startRGBColor;
 uniform highp vec3 endRGBColor;
 uniform highp float strength;

 highp vec3 rgb2hsv(highp vec3 c)
 {
    highp vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    highp vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    highp vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    highp float d = q.x - min(q.w, q.y);
    highp float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
 }

 highp vec3 hsv2rgb(highp vec3 c)
 {
    highp vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    highp vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
 }

 void main()
 {
     highp vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);
     highp vec4 textureColor2 = texture2D(inputImageTexture2, textureCoordinate2);
     highp vec3 hsvColor = rgb2hsv(textureColor.rgb);
     highp float mixAlpha;
     highp vec3 colortest = startRGBColor;
     //why: texture coordinate is from 1 -> 0 in this filter....
     highp float y = 1.0 - textureCoordinate2.y;
     if (y < gradientPosStart) {
         mixAlpha = 0.0;
     } else if (y > gradientPosEnd) {
         mixAlpha = 1.0;
     } else {
         mixAlpha = (y - gradientPosStart) / (gradientPosEnd - gradientPosStart);
     }
     highp vec3 hairRGBColor = mix(startRGBColor.rgb, endRGBColor.rgb, mixAlpha);
     highp vec3 hairHSVColor = rgb2hsv(hairRGBColor.rgb);
     highp vec3 blendedColor = hsv2rgb(vec3(hairHSVColor.r, hairHSVColor.g, hsvColor.b));
     gl_FragColor = vec4(mix(blendedColor, textureColor.rgb, 1.0 - strength * textureColor2.r), textureColor.a);
 }