package com.azeesoft.lib.colorpicker

import android.content.Context
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.drawable.BitmapDrawable
import android.os.AsyncTask
import android.os.Build
import android.support.v4.graphics.drawable.RoundedBitmapDrawable
import android.support.v4.graphics.drawable.RoundedBitmapDrawableFactory
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import com.roidapp.baselib.px
import com.roidapp.imagelib.R
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.Channel
import kotlin.coroutines.CoroutineContext

class SatValPicker : ViewGroup, CoroutineScope {
    val job = SupervisorJob()
    override val coroutineContext: CoroutineContext
        get() = Dispatchers.Main.immediate + job

    private var firstDraw: Boolean = false
    private var pendingUpdateSatVal = false
    var isCanUpdateHexVal = true
    private var minSizePx: Int = 0
    private var mWidth: Int = 0
    private var mHeight: Int = 0
    private var mHue = 0f
    private var mSat = 0f
    private var mVal = 1f
    private var pointerX: Float = 0f
    private var pointerY: Float = 0f
    private var mContext: Context? = null
    private var onColorSelectedListener: OnColorSelectedListener? = null
    private var onColorSelectedUpListener: OnColorSelectedListener? = null
    private var onColorSelectedMoveListener: OnColorSelectedListener? = null
    private var onColorSelectedFromHueListener: OnColorSelectedFromHueListener? = null
    private var thumb: ImageView? = null
    private var colorPickerCompatScrollView: ColorPickerCompatScrollView? = null
    private var colorPickerCompatHorizontalScrollView: ColorPickerCompatHorizontalScrollView? = null

    private var genBitmapChannel = Channel<GenColorMapParam>(Channel.CONFLATED)

    var skipCount = 10
    var lastGenSpendTime: Long = 0

    constructor(context: Context) : super(context) {
        init(context)
    }

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs) {
        init(context)
    }

    constructor(context: Context, attrs: AttributeSet, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        init(context)
    }

    override fun onLayout(changed: Boolean, l: Int, t: Int, r: Int, b: Int) {
        val count = childCount
        var curWidth: Int
        var curHeight: Int
        var curLeft: Int
        var curTop: Int
        var maxHeight: Int

        //get the available size of child view
        val childLeft = this.paddingLeft
        val childTop = this.paddingTop
        val childRight = this.measuredWidth - this.paddingRight
        val childBottom = this.measuredHeight - this.paddingBottom
        val childWidth = childRight - childLeft
        val childHeight = childBottom - childTop

        maxHeight = 0
        curLeft = childLeft
        curTop = childTop

        for (i in 0 until count) {
            val child = getChildAt(i)

            if (child.visibility == View.GONE)
                return

            //Get the maximum size of the child
            child.measure(View.MeasureSpec.makeMeasureSpec(childWidth, View.MeasureSpec.AT_MOST), View.MeasureSpec.makeMeasureSpec(childHeight, View.MeasureSpec.AT_MOST))
            curWidth = child.measuredWidth
            curHeight = child.measuredHeight
            //wrap is reach to the end
            if (curLeft + curWidth >= childRight) {
                curLeft = childLeft
                curTop += maxHeight
                maxHeight = 0
            }
            //do the layout
            child.layout(curLeft, curTop, curLeft + curWidth, curTop + curHeight)
            //store the max height
            if (maxHeight < curHeight)
                maxHeight = curHeight
            curLeft += curWidth
        }
    }


    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        // Constrain to square
        /*
        final int w = MeasureSpec.getSize(widthMeasureSpec);
        final int h = MeasureSpec.getSize(heightMeasureSpec);
        final int modeW = MeasureSpec.getMode(widthMeasureSpec);
        final int modeH = MeasureSpec.getMode(heightMeasureSpec);
        int size;
        if (modeW == MeasureSpec.UNSPECIFIED) {
            size = h;
        } else if (modeH == MeasureSpec.UNSPECIFIED) {
            size = w;
        } else {
            size = Math.min(w, h);
        }
        size = Math.max(size, minSizePx);
        setMeasuredDimension(size, size);
*/

        super.onMeasure(widthMeasureSpec, heightMeasureSpec)

        mWidth = measuredWidth
        mHeight = measuredHeight
        //System.out.println("init sat val measures: " + mWidth + "," + mHeight);
        if (firstDraw) {
            firstDraw = false
            refreshSatValPicker(mHue)
        }
    }

    @Synchronized
    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
    }

    fun init(context: Context) {
        mContext = context
        minSizePx = Stools.dipToPixels(context, MIN_SIZE_DIP.toFloat()).toInt()
        firstDraw = true
        pendingUpdateSatVal = false

        thumb = LayoutInflater.from(mContext).inflate(R.layout.sat_val_thumb, null) as ImageView

        thumb!!.pivotX = Stools.dipToPixels(mContext!!, 6f)
        thumb!!.pivotY = Stools.dipToPixels(mContext!!, 6f)

        addView(thumb)

        monitorEvent()
    }

    private fun monitorEvent(){
        launch {
            for(event in genBitmapChannel){

                val start = System.currentTimeMillis()
                val bitmapDrawable = genSatValBitmapAsync(event.hue, event.w, event.h, event.skipCount).await()
                if(lastGenSpendTime == 0L) {
                    lastGenSpendTime = System.currentTimeMillis() - start
                }else{
                    lastGenSpendTime = (System.currentTimeMillis() - start + lastGenSpendTime)/2
                }

                bitmapDrawable.cornerRadius = 4.px.toFloat()

                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                    background = bitmapDrawable
                } else {
                    setBackgroundDrawable(bitmapDrawable)
                }

                if (pendingUpdateSatVal)
                    placePointer(mSat * mWidth, mHeight - mVal * mHeight, isFromHue = event.isFromHue, fromHuePickerUp = event.fromHuePickerUp)
                else
                    retrieveColorAt(pointerX, pointerY, isFromHue = event.isFromHue, fromHuePickerUp = event.fromHuePickerUp)
            }
        }
    }

    override fun onTouchEvent(event: MotionEvent): Boolean {
        when (event.action) {
            MotionEvent.ACTION_DOWN -> {
                //System.out.println("DOWN\n----");
                placePointer(event.x, event.y)
                disableScroll()
                return true
            }
            MotionEvent.ACTION_MOVE -> {
                //System.out.println("MOVE\n----");
                placePointer(event.x, event.y, isMove = true)
                disableScroll()
                return true
            }
            MotionEvent.ACTION_UP -> {
                retrieveColorAt(event.x, event.y, true)
                enableScroll()
                return true
            }
        }
        enableScroll()
        return false
        //        return super.onTouchEvent(event);
    }

    private fun disableScroll() {
        if (colorPickerCompatScrollView != null)
            colorPickerCompatScrollView!!.setScrollDisabled(true)

        if (colorPickerCompatHorizontalScrollView != null)
            colorPickerCompatHorizontalScrollView!!.setScrollDisabled(true)
    }

    private fun enableScroll() {
        if (colorPickerCompatScrollView != null)
            colorPickerCompatScrollView!!.setScrollDisabled(false)


        if (colorPickerCompatHorizontalScrollView != null)
            colorPickerCompatHorizontalScrollView!!.setScrollDisabled(false)
    }

    private fun placePointer(x: Float, y: Float, isMove: Boolean=false, isFromHue: Boolean = false, fromHuePickerUp: Boolean=false) {
        var x = x
        var y = y
        if (mWidth > 0 && mHeight > 0) {
            if (x < 0)
                x = 0f
            else if (x > mWidth)
                x = mWidth.toFloat()

            if (y < 0)
                y = 0f
            else if (y > mHeight)
                y = mHeight.toFloat()

            thumb!!.x = x - Stools.dipToPixels(mContext!!, 6f)
            thumb!!.y = y - Stools.dipToPixels(mContext!!, 6f)
            if (y < mHeight / 2)
                thumb!!.setImageDrawable(mContext!!.resources.getDrawable(R.drawable.thumb_white_solid))
            else
                thumb!!.setImageDrawable(mContext!!.resources.getDrawable(R.drawable.thumb_white_solid))

            retrieveColorAt(x, y, isMove = isMove, isFromHue = isFromHue, fromHuePickerUp = fromHuePickerUp)

        }
        //System.out.println("AshX: " + x);
        //System.out.println("AshY: " + y);
    }

    private fun retrieveColorAt(x: Float, y: Float, isUp: Boolean = false, isMove: Boolean = false, isFromHue: Boolean = false, fromHuePickerUp: Boolean=false) {

        pointerX = x
        pointerY = y

        mSat = x / mWidth.toFloat()
        mVal = (mHeight - y) / mHeight.toFloat()

        onColorRetrieved(mHue, mSat, mVal, isUp, isMove, isFromHue, fromHuePickerUp)
    }

    private fun onColorRetrieved(hue: Float, sat: Float, value: Float, isUp: Boolean = false, isMove: Boolean = false, isFromHue: Boolean = false, fromHuePickerUp: Boolean=false) {
        val color = Color.HSVToColor(floatArrayOf(hue, sat, value))

        val hexVal = "#" + Integer.toHexString(color)
        if (onColorSelectedListener != null) {
            onColorSelectedListener!!.onColorSelected(color, hexVal)
        }

        if (isUp && onColorSelectedUpListener != null) {
            onColorSelectedUpListener!!.onColorSelected(color, hexVal)
        }

        if (isMove && onColorSelectedMoveListener != null) {
            onColorSelectedMoveListener!!.onColorSelected(color, hexVal)
        }
        if (isFromHue && onColorSelectedFromHueListener != null) {
            onColorSelectedFromHueListener!!.onColorSelected(color, hexVal, fromHuePickerUp)
        }
    }

    fun setOnColorSelectedUpListener(onColorSelectedListener: OnColorSelectedListener) {
        this.onColorSelectedUpListener = onColorSelectedListener
    }

    fun setOnColorSelectedMoveListener(onColorSelectedListener: OnColorSelectedListener) {
        this.onColorSelectedMoveListener = onColorSelectedListener
    }

    fun setOnColorSelectedListener(onColorSelectedListener: OnColorSelectedListener) {
        this.onColorSelectedListener = onColorSelectedListener
    }

    fun setOnColorSelectedFromHueListener(onColorSelectedListener: OnColorSelectedFromHueListener) {
        this.onColorSelectedFromHueListener = onColorSelectedListener
    }

    @JvmOverloads
    fun setSaturationAndValue(sat: Float, value: Float, apply: Boolean = true) {
        //System.out.println("Ash Sat Val: "+ sat+" " + val);
        //System.out.println("Ash mWidth mHeight: "+ mWidth+" "+mHeight);
        if (mWidth <= 0 || mHeight <= 0 || !apply) {
            mSat = sat
            mVal = value
            pendingUpdateSatVal = true
        } else
            placePointer(sat * mWidth, mHeight - value * mHeight)
    }

    private suspend fun genSatValBitmapAsync(hue: Float, w: Int, h: Int, skipCount: Int): Deferred<RoundedBitmapDrawable>{
        return async(Dispatchers.Default) {
            RoundedBitmapDrawableFactory.create(mContext!!.resources, BitmapsGenerator.getSatValBitmap(hue, w, h, skipCount, skipCount)) //Math.max(skipCount-2, 5)
        }
    }

    fun refreshSatValPicker(hue: Float, fromHuePicker: Boolean=false, fromHuePickerUp: Boolean=false) {
        mHue = hue

        if (mWidth <= 0 || mHeight <= 0){
            return
        }

        launch {
            genBitmapChannel.send(GenColorMapParam(hue, mWidth, mHeight, skipCount, fromHuePicker, fromHuePickerUp))
        }

    }

    fun setColorPickerCompatScrollView(colorPickerCompatScrollView: ColorPickerCompatScrollView) {
        this.colorPickerCompatScrollView = colorPickerCompatScrollView
    }

    fun setColorPickerCompatHorizontalScrollView(colorPickerCompatHorizontalScrollView: ColorPickerCompatHorizontalScrollView) {
        this.colorPickerCompatHorizontalScrollView = colorPickerCompatHorizontalScrollView
    }

    interface OnColorSelectedListener {
        fun onColorSelected(color: Int, hexVal: String)
    }

    interface OnColorSelectedFromHueListener {
        fun onColorSelected(color: Int, hexVal: String, selectUp: Boolean)
    }

    companion object {

        private val MIN_SIZE_DIP = 50
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        job.cancel()
    }

    data class GenColorMapParam(val hue: Float, val w: Int, val h: Int, val skipCount: Int, val isFromHue: Boolean = false, val fromHuePickerUp: Boolean=false)

}
