package com.roidapp.imagelib.facedetector.orionvision

import android.graphics.PointF
import android.graphics.RectF
import com.orion.vision.bean.Face
import com.roidapp.imagelib.camera.DetectFacialExpression
import com.roidapp.imagelib.facedetector.FaceDetectorConstants
import com.roidapp.imagelib.facedetector.FaceStructProducerInterface
import com.roidapp.imagelib.retouch.facetrack.FaceDetectLandmark
import comroidapp.baselib.util.Logger

class OrionVisionFaceStructProducer : FaceStructProducerInterface<Face> {

    companion object {
        private val instance by lazy { OrionVisionFaceStructProducer() }

        @JvmStatic
        fun getInstance(): FaceStructProducerInterface<Face> { return instance }
    }


    override fun produce(face: Face): FaceStructProducerInterface.FaceStruct {
        val pointArray = Array(FaceDetectorConstants.FACE_FEATURE_COUNT) { PointF() }
        // since top/bottem is y axis, we must starts from odd index!
        var leftIndex = 0
        var topIndex = 1
        var rightIndex = 0
        var bottomIndex = 1

        face.keypoints.forEachIndexed { i, value ->
            if (i and 1 != 0) { // odd
                if (value < face.keypoints[topIndex]) topIndex = i
                if (value > face.keypoints[bottomIndex]) bottomIndex = i
                pointArray[i shr 1].y = value
            } else { // even
                if (value < face.keypoints[leftIndex]) leftIndex = i
                if (value > face.keypoints[rightIndex]) rightIndex = i
                pointArray[i shr 1].x = value
            }
        }

        if (Logger.ENABLE_LOG) Logger.d("compact rect traversal: l = $leftIndex, t = $topIndex, r = $rightIndex, b = $bottomIndex")

        return FaceStructProducerInterface.FaceStruct(pointArray,
                face.keypoints,
                RectF(face.x1, face.y1, face.x2, face.y2),
                RectF(face.keypoints[leftIndex], face.keypoints[topIndex], face.keypoints[rightIndex], face.keypoints[bottomIndex]))
    }

    override fun produceFaceDetectLandmark(face: Face, width: Int, height: Int, orientation: Int): FaceDetectLandmark? {
        val faceStruct = produce(face)
        val points = faceStruct.points
        val faceRect = faceStruct.compactFaceRect

        var temp: Float
        temp = width - faceRect.left
        faceRect.left = width - faceRect.right
        faceRect.right = temp
        temp = height - faceRect.top
        faceRect.top = height - faceRect.bottom
        faceRect.bottom = temp

        if (Logger.ENABLE_LOG) {
            Logger.d("mapping face from OrionVision faceRect= ${faceStruct.faceRect}, " +
                    "compactFaceRect= ${faceStruct.compactFaceRect}, " +
                    "w = $width,  h = $height")
        }

        if (faceRect.left == 0f && faceRect.right == 0f && faceRect.bottom == 0f && faceRect.top == 0f) {
            return null
        }

        val isMouthOpened = DetectFacialExpression.getInstance().detectMouthIsOpen(face)

        if (Logger.ENABLE_LOG) {
            val builder = StringBuilder()
            var count = 0
            for (p in points) {
                builder.append(" (" + count + " " + p.x + ", " + p.y + ")")
                ++count
            }
            Logger.d("mapping face from OrionVision landmarks $builder")
        }

        return FaceDetectLandmark(width, height, faceRect, points, isMouthOpened, orientation)
    }

    private fun getPoint(face: Face, index: Int): PointF {
        return when(index >= 0) {
            true -> PointF(face.keypoints[index shl 1], face.keypoints[(index shl 1) + 1])
            false -> PointF(0.0f, 0.0f)
        }
    }
}