package com.roidapp.imagelib.filter

import android.arch.lifecycle.*
import com.roidapp.baselib.resources.ResourcesAction
import com.roidapp.baselib.resources.ResourcesActionLisenter
import com.roidapp.baselib.utils.isNotNull
import com.roidapp.baselib.utils.isNull
import com.roidapp.imagelib.apis.FilterApiViewModelResult
import com.roidapp.imagelib.resources.filter.FilterGroupInfoList
import com.roidapp.imagelib.resources.filter.FilterResourceManager
import com.roidapp.photogrid.points.exceptions.ApiExceptionUtils
import comroidapp.baselib.util.Logger
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.lang.Exception
import java.lang.ref.WeakReference
import kotlin.coroutines.resume
import kotlin.coroutines.resumeWithException
import kotlin.coroutines.suspendCoroutine

class FilterLoaderViewMode : ViewModel(), LifecycleOwner {
    private val lifecycleRegistry = LifecycleRegistry(this)
    var observerFilterGroupInfoList = MutableLiveData<FilterApiViewModelResult>()

    init {
        GlobalScope.launch(Dispatchers.Main) {
            //run on main...
            lifecycleRegistry.markState(Lifecycle.State.CREATED)
            lifecycleRegistry.markState(Lifecycle.State.STARTED)
            lifecycleRegistry.markState(Lifecycle.State.RESUMED)

        }
    }

    override fun getLifecycle(): Lifecycle {
        return lifecycleRegistry
    }

    fun loadFilter(){

        Logger.dt("loadFilter")
        GlobalScope.launch(Dispatchers.Main) {
            Logger.dt("loadFilter2")
            val result = withContext(Dispatchers.IO) {
                try {
                    val result = loadFilterDataSuspend()
                    Logger.dt("loadFilter2 done")
                    FilterApiViewModelResult.SUCCESS(result)
                } catch (e: Throwable) {
                    val apiException = ApiExceptionUtils.getApiException(e)
                    FilterApiViewModelResult.ERROR(apiException.errorCode, apiException)
                }
            }
            result?.run { observerFilterGroupInfoList.value = result }
            Logger.dt("loadFilter after withcontext")
        }

    }

    private suspend fun loadFilterDataSuspend() = suspendCoroutine<FilterGroupInfoList> { cont ->

        Logger.dt("loadFilter 3")
        FilterResourceManager.getInstance().getInfoList(ResourcesAction.ACTION_TYPE_COMMON_WITH_CACHE, 0, 20, true, FilterGroupInfoList::class.java, object : ResourcesActionLisenter<FilterGroupInfoList> {
            override fun getOK(result: FilterGroupInfoList?) {
                Logger.dt("loadFilter 4")
                result isNotNull {cont.resume(result!!)}
                       isNull {cont.resumeWithException(exception = Throwable("failed"))}
            }

            override fun getFailed(responseCode: Int, e: java.lang.Exception?) {
                Logger.dt("loadFilter 5")
                cont.resumeWithException(exception = Throwable("failed"))
            }

            override fun getCancel() {
                Logger.dt("loadFilter 6")
                cont.resumeWithException(Throwable("failed"))
            }

        })

    }

    override fun onCleared() {
        super.onCleared()
        GlobalScope.launch(Dispatchers.Main) {
            lifecycleRegistry.markState(Lifecycle.State.DESTROYED)
        }
    }

}