package com.roidapp.imagelib.filter

import com.roidapp.baselib.release.Resources.*
import com.roidapp.imagelib.common.ImageColour
import com.roidapp.imagelib.filter.filterinfo.IFilterInfo
import com.roidapp.imagelib.filter.groupinfo.IGroupInfoUtil
import com.roidapp.imagelib.resources.filter.FilterGroupInfo

/**
 * Created by chenlong on 2015/1/12.
 */
object GlobalFilter {

    /**
     * the global filter cache info
     */
    private var globalGroupInfo: FilterGroupInfo? = null
    private var globalImageProperty: ImageColour.ImageProperty? = null
    private var filterInfo: IFilterInfo? = null
    var sketchFilterType: Int = 0

    private val numberValueHelper = NumberValueHelper<FilterType.FilterParamType>()


    init {
        numberValueHelper.delcareValueType(FilterType.FilterParamType.ALPHA_BLEND, ValueType.INT)
        numberValueHelper.delcareValueType(FilterType.FilterParamType.GLITCH_OFFSET, ValueType.FLOAT)
        numberValueHelper.delcareValueType(FilterType.FilterParamType.GLITCH2_OFFSET, ValueType.FLOAT)
        numberValueHelper.delcareValueType(FilterType.FilterParamType.GLITCH3_OFFSET, ValueType.FLOAT)

        numberValueHelper.setFilterParamValue(FilterType.FilterParamType.ALPHA_BLEND, DEFAULT_ALPHA_BLEND_PROGRESS)
        numberValueHelper.setFilterParamValue(FilterType.FilterParamType.GLITCH_OFFSET, DEFAULT_GLITCH_PROGRESS)
        numberValueHelper.setFilterParamValue(FilterType.FilterParamType.GLITCH2_OFFSET, DEFAULT_GLITCH2_PROGRESS)
        numberValueHelper.setFilterParamValue(FilterType.FilterParamType.GLITCH3_OFFSET, DEFAULT_GLITCH3_PROGRESS)

        globalImageProperty = ImageColour.ImageProperty()
    }

    val isProcessGlobalFilter: Boolean
        get() = synchronized(this) {
            if (globalImageProperty!!.hasChange()) return true
            return if (IGroupInfoUtil.hasSelFilterInfo(globalGroupInfo) && globalGroupInfo!!.selFilterInfo.filterType != FilterType.ORIGINAL || filterInfo != null && filterInfo!!.filterType != FilterType.ORIGINAL) true else false
        }


    fun setFilterParamValue(type: FilterType.FilterParamType, value: Number) {
        synchronized(this) {
            numberValueHelper.setFilterParamValue(type, value)
        }
    }

    fun <T> getFilterParamValue(type: FilterType.FilterParamType): T {
        synchronized(this) {
            return numberValueHelper.getFilterParamValue(type)
        }
    }

    fun getAlphaProgress(): Int {
        synchronized(this) {
            return getFilterParamValue(FilterType.FilterParamType.ALPHA_BLEND)
        }
    }

    fun setAlphaProgress(alphaProgress: Int) {
        synchronized(this) {
            setFilterParamValue(FilterType.FilterParamType.ALPHA_BLEND, alphaProgress)
        }
    }

    fun getFilterInfo(): IFilterInfo? {
        synchronized(this) {
            return filterInfo
        }
    }

    fun setGlobalFilterInfo(filterInfo: IFilterInfo?) {
        synchronized(this) {
            this.filterInfo = filterInfo
        }
    }

    fun getGlobalGroupInfo(): FilterGroupInfo? {
        synchronized(this) {
            return globalGroupInfo
        }
    }

    fun setGlobalGroupInfo(globalGroupInfo: FilterGroupInfo?) {
        synchronized(this) {
            this.globalGroupInfo = globalGroupInfo
        }
    }

    fun getGlobalImageProperty(): ImageColour.ImageProperty? {
        synchronized(this) {
            return globalImageProperty
        }
    }

    fun setGlobalImageProperty(globalImageProperty: ImageColour.ImageProperty?) {
        synchronized(this) {
            this.globalImageProperty = globalImageProperty
        }
    }

    @Synchronized
    fun reset() {
        synchronized(this) {
            globalGroupInfo = null
            filterInfo = null
            globalImageProperty = ImageColour.ImageProperty()
        }
    }

}
