package com.roidapp.imagelib.filter.producer.impl

import android.content.Context
import com.roidapp.imagelib.filter.filterinfo.CloudFilterInfo
import com.roidapp.imagelib.filter.producer.FilterProducerBase
import com.roidapp.imagelib.filter.producer.Params
import comroidapp.baselib.util.Logger
import jp.co.cyberagent.android.gpuimage.GPUImageFilter
import jp.co.cyberagent.android.gpuimage.GPUImageGlitch2Filter
import java.io.InputStream
import java.lang.Exception
import java.util.ArrayList

class GlitchFilterProducer2(private val context: Context,
                           private val mCfi: CloudFilterInfo) : FilterProducerBase() {

    override fun produceFilter(params: Params): List<GPUImageFilter>? {
        val filters = ArrayList<GPUImageFilter>()

        var input: InputStream? = null
        try {
            input = context.assets.open("glsl/glitch2_fragment_shader.glsl")
            val glsl = input.readBytes()?.toString(Charsets.UTF_8)
            val glitchDirection = mCfi.extra.getString(CloudFilterInfo.Keys.GLITCH_DIRECTION, "1").toInt()
            val glitchColorMode = mCfi.extra.getString(CloudFilterInfo.Keys.GLITCH_COLOR_MODE, "1").toInt()

            if (Logger.ENABLE_LOG) {
                Logger.d("GlitchFilterProducer2 called by id " + mCfi.id + " with:" +
                        " name = [" + mCfi.mName + "], " +
                        " glitch_direction = [" + glitchDirection + "]" +
                        " glitch_color_mode = [" + glitchColorMode + "]")
            }

            val glitchFilter = GPUImageGlitch2Filter(glsl)
            glitchFilter.colorMode = glitchColorMode
            filters.add(glitchFilter)
        }
        catch (e : Exception){
            e.printStackTrace()
        }
        finally {
            input?.run {
                this.close()
            }
        }

        return filters
    }
}