package com.roidapp.imagelib.filter.producer.impl

import android.graphics.Bitmap
import android.text.TextUtils
import com.roidapp.imagelib.common.ImageLoader
import com.roidapp.imagelib.filter.filterinfo.CloudFilterInfo
import com.roidapp.imagelib.filter.producer.FilterProducerBase
import com.roidapp.imagelib.filter.producer.Params
import comroidapp.baselib.util.Logger
import jp.co.cyberagent.android.gpuimage.GPUImageFilter
import jp.co.cyberagent.android.gpuimage.GPUImageSkyBlendFilter
import java.util.*

class SkyBlendFilterProducer(private val mCfi: CloudFilterInfo,
                             private var imageRatio:Float) : FilterProducerBase() {

    override fun produceFilter(params: Params): List<GPUImageFilter>? {
        val filters = ArrayList<GPUImageFilter>()
        val minSideLength = params.get(Params.Keys.MIN_SIDE_LEN) as Int
        val maxNumOfPixels = params.get(Params.Keys.PIXEL) as Int
        val extra = mCfi.extra

        //# Texture
        var filterTexture: Bitmap? = null
        val textureFile = extra.getString(CloudFilterInfo.Keys.TEXTURE)
        if (!TextUtils.isEmpty(textureFile)) {
            val texturePath = mCfi.getFilePathFromExtra(textureFile)
            if (Logger.ENABLE_LOG) {
                Logger.d("produceFilter texturePath: $texturePath")
            }
            filterTexture = ImageLoader.decodeStream(texturePath, minSideLength, maxNumOfPixels)
        }

        //# param
        val temperatureStr = extra.getString(CloudFilterInfo.Keys.TEMPERATURE)
        val gammaStr = extra.getString(CloudFilterInfo.Keys.GAMMA)
        var temperature = 5000f
        var gama = 1.0f
        temperatureStr?.run {
            temperature = parseFloat(temperatureStr)
        }

        gammaStr?.run {
            gama = parseFloat(gammaStr)
        }

        if (Logger.ENABLE_LOG) {
            Logger.d("produceFilter temperature: $temperature, gamma:$gama")
        }
        val skyBlendFilter = GPUImageSkyBlendFilter(1.0f, temperature, gama, imageRatio)
        skyBlendFilter.bitmap = filterTexture
        filters.add(skyBlendFilter)
        return filters
    }
}