package jp.co.cyberagent.android.gpuimage


import android.opengl.GLES20

class GPUImageGlitch2Filter(GLITCH2_FRAGMENT_SHADER: String) : GPUImageFilter(NO_FILTER_VERTEX_SHADER, GLITCH2_FRAGMENT_SHADER) {

    val TAG = "Glitch2"
    private var directionLocation: Int? = null
    private var colorModeLocation: Int? = null
    private var offsetLocation: Int? = null
    private var VDisplacementLocation: Int? = null
    private var phaseShiftLocation: Int? = null
    private var amplitudeLocation: Int? = null
    private var noiseScaleLocation: Int? = null

    var mDirection: Int=0
    var mOffset: Float=0.0f
    var mAmplitude: Float=0.0f
    var mPhaseShift: Float=0.0f
    var mVDisplacement: Float=0.0f
    var mNoiseScale: Float=0.0f

    //val propertyMap = HashMap<String, Pair<>>

    var colorMode: Int=0
        set(value) {
            field = value
            colorModeLocation?.run {
                setInteger(this, value)
            }
        }

    init {

        mDirection = 1
        colorMode = 2
        mOffset = 0.0f
        mAmplitude = 1.0f
        mPhaseShift = 0.0f
        mVDisplacement = 0.0f
        mNoiseScale = 0.05f
    }

    override fun onInit() {
        super.onInit()
        directionLocation = GLES20.glGetUniformLocation(program, "direction")
        colorModeLocation = GLES20.glGetUniformLocation(program, "colorMode")
        offsetLocation = GLES20.glGetUniformLocation(program, "offset")
        VDisplacementLocation = GLES20.glGetUniformLocation(program, "vDisplacement")
        phaseShiftLocation = GLES20.glGetUniformLocation(program, "phaseShift")
        amplitudeLocation = GLES20.glGetUniformLocation(program, "amplitude")
        noiseScaleLocation = GLES20.glGetUniformLocation(program, "noiseScale")
    }

    override fun onInitialized() {
        super.onInitialized()

        setDirection(mDirection)
        colorMode = colorMode
        setOffset(mOffset)
        setPhaseShift(mPhaseShift)
        setAmplitude(mAmplitude)
        setVDisplacement(mVDisplacement)
        setNoiseScale(mNoiseScale)
    }

    fun setDirection(direction: Int) {
        mDirection = direction
        directionLocation?.run { setInteger(this, mDirection) }

    }

    fun setOffset(offset: Float) {
        mOffset = offset
        offsetLocation?.run { setFloat(this, mOffset) }

    }
    fun setVDisplacement(vDisplacement: Float) {
        mVDisplacement = vDisplacement
        VDisplacementLocation?.run { setFloat(this, mVDisplacement) }

    }
    fun setPhaseShift(phaseShift: Float) {
        mPhaseShift = phaseShift
        phaseShiftLocation?.run { setFloat(this, mPhaseShift) }

    }
    fun setAmplitude(amplitude: Float) {
        mAmplitude = amplitude
        amplitudeLocation?.run { setFloat(this, mAmplitude) }

    }
    fun setNoiseScale(noiseScale: Float) {
        mNoiseScale = noiseScale
        noiseScaleLocation?.run { setFloat(this, mNoiseScale) }

    }

}
