package jp.co.cyberagent.android.gpuimage

import android.opengl.GLES20

class GPUImageGlitchFilter : GPUImageFilter {

    companion object {
        val DIRECTION_HORIZONTAL = 1
        val DIRECTION_VERTICAL = 2

        val COLOR_MODE_1 = 1
        val COLOR_MODE_2 = 2
        val COLOR_MODE_3 = 3
        val COLOR_MODE_4 = 4
        val COLOR_MODE_5 = 5
        val COLOR_MODE_6 = 6

        val OFFSET_DEFAULT = 0.15f

        private val GLITCH_FRAGMENT_SHADER = "" +
                "varying highp vec2 textureCoordinate;" +
                "" +
                "uniform sampler2D inputImageTexture;" +
                "" +
                "uniform int direction;" +
                "uniform int colorMode;" +
                "uniform lowp float offset;" +
                "" +
                "highp float shiftedX;" +
                "highp float shiftedY;" +
                "highp float shiftedX2;" +
                "highp float shiftedY2;" +
                "" +
                "highp float fixShiftedValue(highp float shiftedValue) {" +
                "    if (shiftedValue > 1.0) {" +
                "        return 1.0;" +
                "    } else if (shiftedValue < 0.0) {" +
                "        return 0.0;" +
                "    }" +
                "    return shiftedValue;" +
                "}" +
                "" +
                "void main() {" +
                "    shiftedX = textureCoordinate.x;" +
                "    shiftedY = textureCoordinate.y;" +
                "" +
                "    shiftedX2 = textureCoordinate.x;" +
                "    shiftedY2 = textureCoordinate.y;" +
                "" +
                "    if (direction == 1) {" +
                "        shiftedX = textureCoordinate.x + offset;" +
                "        shiftedX2 = textureCoordinate.x - offset;" +
                "" +
                "        shiftedX = fixShiftedValue(shiftedX);" +
                "        shiftedX2 = fixShiftedValue(shiftedX2);" +
                "    } else if (direction == 2) {" +
                "        shiftedY = textureCoordinate.y + offset;" +
                "        shiftedY2 = textureCoordinate.y - offset;" +
                "" +
                "        shiftedY = fixShiftedValue(shiftedY);" +
                "        shiftedY2 = fixShiftedValue(shiftedY2);" +
                "    }" +
                "" +
                "    lowp vec4 textureColor = texture2D(inputImageTexture, vec2(shiftedX, shiftedY));" +
                "    lowp vec4 textureColor2 = texture2D(inputImageTexture, vec2(shiftedX2, shiftedY2));" +
                "" +
                "    if ( 1 <= colorMode && colorMode <= 3) {" +
                "        lowp vec4 oriTextureColor = texture2D(inputImageTexture, textureCoordinate);" +
                "        if (colorMode == 1) {" +
                "            gl_FragColor = vec4(textureColor.r, oriTextureColor.g, textureColor2.b, 1.0);" +
                "        } else if (colorMode == 2) {" +
                "            gl_FragColor = vec4(textureColor.r, textureColor2.g, oriTextureColor.b, 1.0);" +
                "        } else if (colorMode == 3) {" +
                "            gl_FragColor = vec4(oriTextureColor.r, textureColor.g, textureColor2.b, 1.0);" +
                "        }" +
                "    } else {" +
                "        if (colorMode == 4) {" +
                "            gl_FragColor = vec4(textureColor2.r, textureColor.g, textureColor2.b, 1.0);" +
                "        } else if (colorMode == 5) {" +
                "            gl_FragColor = vec4(textureColor.r, textureColor.g, textureColor2.b, 1.0);" +
                "        } else if (colorMode == 6) {" +
                "            gl_FragColor = vec4(textureColor2.r, textureColor.g, textureColor.b, 1.0);" +
                "        } else {" +
                "            gl_FragColor = textureColor;" +
                "        }" +
                "    }" +
                "}"
    }

    private var mDirectionLocation: Int = 0
    private var mDirection: Int = 0
    private var mColorModeLocation: Int = 0
    private var mColorMode: Int = 0
    private var mOffsetLocation: Int = 0
    private var mOffset: Float = 0.toFloat()

    constructor() : this(DIRECTION_HORIZONTAL, COLOR_MODE_1, OFFSET_DEFAULT)
    constructor(direction: Int, colorMode: Int) : this(direction, colorMode, OFFSET_DEFAULT)
    constructor(direction: Int, colorMode: Int, offset: Float) : super(NO_FILTER_VERTEX_SHADER, GLITCH_FRAGMENT_SHADER) {
        mDirection = direction
        mColorMode = colorMode
        mOffset = offset
    }

    override fun onInit() {
        super.onInit()
        mDirectionLocation = GLES20.glGetUniformLocation(program, "direction")
        mColorModeLocation = GLES20.glGetUniformLocation(program, "colorMode")
        mOffsetLocation = GLES20.glGetUniformLocation(program, "offset")
        setDirection(mDirection)
        setColorMode(mColorMode)
        setOffset(mOffset)
    }

    fun setDirection(direction: Int) {
        mDirection = direction
        setInteger(mDirectionLocation, direction)
    }

    fun setColorMode(colorMode: Int) {
        mColorMode = colorMode
        setInteger(mColorModeLocation, colorMode)
    }

    fun setOffset(offset: Float) {
        mOffset = offset
        setFloat(mOffsetLocation, offset)
    }
}